import { validateWithTypeBox } from '@hello.nrfcloud.com/proto';
import {} from '@sinclair/typebox';
import { toFetchError } from './FetchError.js';
import { slashless } from './slashless.js';
export class ValidationError extends Error {
    errors;
    isValidationError = true;
    constructor(errors) {
        super(`Validation errors`);
        this.name = 'ValidationError';
        this.errors = errors;
    }
}
const validate = (SchemaObject, data) => {
    const maybeData = validateWithTypeBox(SchemaObject)(data);
    if ('errors' in maybeData) {
        console.error('Validation failed', { error: maybeData.errors });
        throw new ValidationError(maybeData.errors);
    }
    return maybeData.value;
};
const fetchData = (fetchImplementation) => async (...args) => {
    const response = await (fetchImplementation ?? fetch)(...args);
    if (!response.ok)
        throw await toFetchError(response);
    if (response.headers?.get('content-length') === '0')
        return undefined;
    return response.json();
};
export const validatedFetch = ({ endpoint, apiKey }, fetchImplementation) => async (params, schema) => {
    const { resource, query } = params;
    const args = {
        headers: headers(apiKey),
    };
    if ('payload' in params) {
        const payload = params.payload;
        args.method = 'POST';
        args.body = payload.body;
        args.headers = { ...(args.headers ?? {}), ['Content-Type']: payload.type };
    }
    else if ('method' in params) {
        args.method = params.method;
    }
    return fetchData(fetchImplementation)(`${slashless(endpoint)}/v1/${resource}${query !== undefined ? `?${query.toString()}` : ''}`, args)
        .then((res) => ({ result: validate(schema, res) }))
        .catch((error) => ({
        error,
    }));
};
const headers = (apiKey) => ({
    Authorization: `Bearer ${apiKey}`,
    Accept: 'application/json; charset=utf-8',
});
export const JSONPayload = (payload) => ({
    type: 'application/json',
    body: JSON.stringify(payload),
});
